# -*- coding: utf-8 -*-
import tkinter as tk
import customtkinter
from tkinter import filedialog as fd
from functools import partial
import MistrasDTA
import os
import gc
import numpy as np
import pandas as pd
from tkinter import messagebox
import matplotlib.pyplot as plt
from matplotlib.backends.backend_tkagg import (
    FigureCanvasTkAgg, NavigationToolbar2Tk)
from numpy.lib.recfunctions import join_by

customtkinter.set_appearance_mode("dark")  # Modes: system (default), light, dark
customtkinter.set_default_color_theme("blue")  # Themes: blue (default), dark-blue, green
    
root = customtkinter.CTk()  
root.tk.call('tk', 'scaling', 1.4)
root.geometry('1230x880')     
root.resizable(width=False, height=False)
root.title('DTA event waveviewer')  
root.configure(bg='#212325')
plt.style.use(['default', 'dark_background'])
plt.rcParams['axes.facecolor'] = '#212325'
plt.rcParams['figure.facecolor'] = '#333333'
plt.rcParams['figure.edgecolor'] = '#2A2A2A'

counter = tk.IntVar()

#-- Functions --#
def _clear():
    for item in canvas.get_tk_widget().find_all():
       canvas.get_tk_widget().delete(item)

def rmfield( a, *fieldnames_to_remove ):
    return a[ [ name for name in a.dtype.names if name not in fieldnames_to_remove ] ]

def loadfile(label_loadfile, label_loadpath):
    filetypes=(("DTA", ".dta"),)
    filename = fd.askopenfilename(
        title='Open a file',
        initialdir='/',
        filetypes=filetypes)
    if filename is None:
        return
    path = os.path.dirname(filename)
    label_loadfile.config(text="%s" % filename)
    label_loadpath.config(text="%s" % path)
    return 

def extractall(label_numhits, label_numevents):
    if not os.path.exists(label_loadDTA.cget("text")):
        return
    
    global AECh
    # Read the binary file
    rec, wfm = MistrasDTA.read_bin(label_loadDTA.cget("text"))
    merged = join_by(['SSSSSSSS.mmmuuun', 'CH'], rec, wfm)
    
    #Convert to Dataframe (Pandas)
    merged_array = merged.view(np.recarray)
    merged_df_full = pd.DataFrame(merged_array)
    merged_df = merged_df_full
    #Remove Channel of choice
    if int(float(Chnum.get())) == 0:
        merged_df = merged_df_full
    else:
        merged_df = merged_df_full.drop(merged_df_full[merged_df_full.CH == int(float(Chnum.get()))].index)

    #Count unique number of channels
    CountCh = merged_df.CH.unique().size
    
    #Set mask with precision on event identification to x number of decimals in time
    merged_round = np.around(merged_df.iloc[:,0], decimals=int(prec.get()))
    merged_join = pd.merge(merged_round, merged_df, left_index=True, right_index=True, how="outer")

    #Extract events on x channels
    Events_ = merged_join.groupby('SSSSSSSS.mmmuuun_x').filter(lambda x: len(x) == CountCh)
    Events = Events_.sort_values(by=['SSSSSSSS.mmmuuun_x', 'CH'])
    
    #Convert back to Record array
    AECh = Events.to_records()
    if len(AECh) == 0:
        fig = plt.figure(figsize=(16,9)  , dpi=72)
        gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
        (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots()
        for ax in fig.get_axes():
            ax.label_outer()
        fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
        fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
        fig.tight_layout(pad=5)

        canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
        canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
            
        toolbarFrame = tk.Frame(master=root)
        toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
        toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
        toolbar.config(bg="#212325")
        toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
        toolbar.winfo_children()[-2].config(background="#212325")
        toolbar.update()
        
        tk.messagebox.showwarning(title="No events!", message="Based on the chosen time precision, no events could be found. Try a lower value. If applicable, you may consider excluding the channel with the cBN anvil due to arrival time differences with the other WC anvils.")
        
        return

    AECh = np.sort(AECh, order=['SSSSSSSS.mmmuuun_x', 'CH'])
    
    if CountCh == 6: 
        t0, V0 = MistrasDTA.get_waveform_data(AECh[0])
        t1, V1 = MistrasDTA.get_waveform_data(AECh[1])
        t2, V2 = MistrasDTA.get_waveform_data(AECh[2])
        t3, V3 = MistrasDTA.get_waveform_data(AECh[3])
        t4, V4 = MistrasDTA.get_waveform_data(AECh[4])
        t5, V5 = MistrasDTA.get_waveform_data(AECh[5])
        
        Ch0 = AECh['CH'][0]
        Ch1 = AECh['CH'][1]
        Ch2 = AECh['CH'][2]
        Ch3 = AECh['CH'][3]
        Ch4 = AECh['CH'][4]
        Ch5 = AECh['CH'][5]
    
        Amp0 = AECh['AMP'][0]
        Amp1 = AECh['AMP'][1]
        Amp2 = AECh['AMP'][2]
        Amp3 = AECh['AMP'][3]
        Amp4 = AECh['AMP'][4]
        Amp5 = AECh['AMP'][5]
        
        En0 = AECh['ABS-ENERGY'][0]
        En1 = AECh['ABS-ENERGY'][1]
        En2 = AECh['ABS-ENERGY'][2]
        En3 = AECh['ABS-ENERGY'][3]
        En4 = AECh['ABS-ENERGY'][4]
        En5 = AECh['ABS-ENERGY'][5]
        
        Count0 = AECh['COUN'][0]
        Count1 = AECh['COUN'][1]
        Count2 = AECh['COUN'][2]
        Count3 = AECh['COUN'][3]
        Count4 = AECh['COUN'][4]
        Count5 = AECh['COUN'][5]
    
        Freq0 = AECh['P-FRQ'][0]
        Freq1 = AECh['P-FRQ'][1]
        Freq2 = AECh['P-FRQ'][2]
        Freq3 = AECh['P-FRQ'][3]
        Freq4 = AECh['P-FRQ'][4]
        Freq5 = AECh['P-FRQ'][5]
        
        Time0 = AECh['SSSSSSSS.mmmuuun_y'][0]
        Time1 = AECh['SSSSSSSS.mmmuuun_y'][1]
        Time2 = AECh['SSSSSSSS.mmmuuun_y'][2]
        Time3 = AECh['SSSSSSSS.mmmuuun_y'][3]
        Time4 = AECh['SSSSSSSS.mmmuuun_y'][4]
        Time5 = AECh['SSSSSSSS.mmmuuun_y'][5]
    
        Thr = AECh['THR'][0]
    
        plt.cla()
        plt.clf()    
        plt.close()
        fig = plt.figure(figsize=(16,9)  , dpi=72)
        gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
        (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
        for ax in fig.get_axes():
            ax.label_outer()
        
        ax1.plot(t0, V0, 'tab:red')
        ax2.plot(t1, V1, 'tab:orange')
        ax3.plot(t2, V2, 'tab:green')
        ax4.plot(t3, V3, 'tab:blue')
        ax5.plot(t4, V4, 'tab:purple')
        ax6.plot(t5, V5, 'tab:pink')
        
        fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
        fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
            
        fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 3.1, "Viewing hits: 1 - 6 | Event: 1", fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.text(-0.98, 2.08, "Arrival time: {:.8f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.08, "Arrival time: {:.8f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.08, "Arrival time: {:.8f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.08, "Arrival time: {:.8f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.08, "Arrival time: {:.8f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 0.08, "Arrival time: {:.8f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.tight_layout(pad=5)
        
        canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
        canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
        
        toolbarFrame = tk.Frame(master=root)
        toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
        toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
        toolbar.config(bg="#212325")
        toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
        toolbar.winfo_children()[-2].config(background="#212325")
        toolbar.update()
        
        numhits = len(AECh)
        label_numhits.config(text="%s" % numhits) 
        numevents = int(numhits / 6)
        label_numevents.config(text="%s" % numevents) 
    else:
        t0, V0 = MistrasDTA.get_waveform_data(AECh[0])
        t1, V1 = MistrasDTA.get_waveform_data(AECh[1])
        t2, V2 = MistrasDTA.get_waveform_data(AECh[2])
        t3, V3 = MistrasDTA.get_waveform_data(AECh[3])
        t4, V4 = MistrasDTA.get_waveform_data(AECh[4])
        #t5, V5 = MistrasDTA.get_waveform_data(AECh[5])
        
        Ch0 = AECh['CH'][0]
        Ch1 = AECh['CH'][1]
        Ch2 = AECh['CH'][2]
        Ch3 = AECh['CH'][3]
        Ch4 = AECh['CH'][4]
        #Ch5 = AECh['CH'][5]
    
        Amp0 = AECh['AMP'][0]
        Amp1 = AECh['AMP'][1]
        Amp2 = AECh['AMP'][2]
        Amp3 = AECh['AMP'][3]
        Amp4 = AECh['AMP'][4]
        #Amp5 = AECh['AMP'][5]
        
        En0 = AECh['ABS-ENERGY'][0]
        En1 = AECh['ABS-ENERGY'][1]
        En2 = AECh['ABS-ENERGY'][2]
        En3 = AECh['ABS-ENERGY'][3]
        En4 = AECh['ABS-ENERGY'][4]
        #En5 = AECh['ABS-ENERGY'][5]
        
        Count0 = AECh['COUN'][0]
        Count1 = AECh['COUN'][1]
        Count2 = AECh['COUN'][2]
        Count3 = AECh['COUN'][3]
        Count4 = AECh['COUN'][4]
        #Count5 = AECh['COUN'][5]
    
        Freq0 = AECh['P-FRQ'][0]
        Freq1 = AECh['P-FRQ'][1]
        Freq2 = AECh['P-FRQ'][2]
        Freq3 = AECh['P-FRQ'][3]
        Freq4 = AECh['P-FRQ'][4]
        #Freq5 = AECh['P-FRQ'][5]
        
        Time0 = AECh['SSSSSSSS.mmmuuun_y'][0]
        Time1 = AECh['SSSSSSSS.mmmuuun_y'][1]
        Time2 = AECh['SSSSSSSS.mmmuuun_y'][2]
        Time3 = AECh['SSSSSSSS.mmmuuun_y'][3]
        Time4 = AECh['SSSSSSSS.mmmuuun_y'][4]
        #Time5 = AECh['SSSSSSSS.mmmuuun_y'][5]
    
        Thr = AECh['THR'][0]
    
        plt.cla()
        plt.clf()    
        plt.close()
        fig = plt.figure(figsize=(16,9)  , dpi=72)
        gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
        (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
        for ax in fig.get_axes():
            ax.label_outer()
        
        ax1.plot(t0, V0, 'tab:red')
        ax2.plot(t1, V1, 'tab:orange')
        ax3.plot(t2, V2, 'tab:green')
        ax4.plot(t3, V3, 'tab:blue')
        ax5.plot(t4, V4, 'tab:purple')
        #ax6.plot(t5, V5, 'tab:pink')
        
        fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
        fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
            
        fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        #fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 3.1, "Viewing hits: 1 - 6 | Event: 1", fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.text(-0.98, 2.08, "Arrival time: {:.8f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.08, "Arrival time: {:.8f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.08, "Arrival time: {:.8f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.08, "Arrival time: {:.8f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.08, "Arrival time: {:.8f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        #fig.text(0.03, 0.08, "Arrival time: {:.8f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.tight_layout(pad=5)
        
        canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
        canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
        
        toolbarFrame = tk.Frame(master=root)
        toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
        toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
        toolbar.config(bg="#212325")
        toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
        toolbar.winfo_children()[-2].config(background="#212325")
        toolbar.update()
        
        numhits = len(AECh)
        label_numhits.config(text="%s" % numhits) 
        numevents = int(numhits / 5)
        label_numevents.config(text="%s" % numevents)         
        
    return

def onClickdec(event=None):
    counter.set(counter.get() - 1)

def onClick(event=None):
    counter.set(counter.get() + 1)
    
def reset(event=None):
    counter.set(0)

def gotobutton(event=None):
    counter.set(int(goto.get()))

def arrow():
    gc.collect() 
    
    if not os.path.exists(label_loadDTA.cget("text")):
        return
    
    #Count unique number of channels
    CountCh = np.unique(AECh['CH']).size

    if CountCh == 6:     
        num = label_counter.cget("text") * 6
        viewfirst = num + 1
        viewlast = num + 6
        eventnum = int(viewlast / 6)
    
        t0, V0 = MistrasDTA.get_waveform_data(AECh[num])
        t1, V1 = MistrasDTA.get_waveform_data(AECh[num + 1])
        t2, V2 = MistrasDTA.get_waveform_data(AECh[num + 2])
        t3, V3 = MistrasDTA.get_waveform_data(AECh[num + 3])
        t4, V4 = MistrasDTA.get_waveform_data(AECh[num + 4])
        t5, V5 = MistrasDTA.get_waveform_data(AECh[num + 5])
        
        Ch0 = AECh['CH'][num]
        Ch1 = AECh['CH'][num + 1]
        Ch2 = AECh['CH'][num + 2]
        Ch3 = AECh['CH'][num + 3]
        Ch4 = AECh['CH'][num + 4]
        Ch5 = AECh['CH'][num + 5]
        
        Amp0 = AECh['AMP'][num]
        Amp1 = AECh['AMP'][num + 1]
        Amp2 = AECh['AMP'][num + 2]
        Amp3 = AECh['AMP'][num + 3]
        Amp4 = AECh['AMP'][num + 4]
        Amp5 = AECh['AMP'][num + 5]
        
        En0 = AECh['ABS-ENERGY'][num]
        En1 = AECh['ABS-ENERGY'][num + 1]
        En2 = AECh['ABS-ENERGY'][num + 2]
        En3 = AECh['ABS-ENERGY'][num + 3]
        En4 = AECh['ABS-ENERGY'][num + 4]
        En5 = AECh['ABS-ENERGY'][num + 5]
        
        Count0 = AECh['COUN'][num]
        Count1 = AECh['COUN'][num + 1]
        Count2 = AECh['COUN'][num + 2]
        Count3 = AECh['COUN'][num + 3]
        Count4 = AECh['COUN'][num + 4]
        Count5 = AECh['COUN'][num + 5]
    
        Freq0 = AECh['P-FRQ'][num]
        Freq1 = AECh['P-FRQ'][num + 1]
        Freq2 = AECh['P-FRQ'][num + 2]
        Freq3 = AECh['P-FRQ'][num + 3]
        Freq4 = AECh['P-FRQ'][num + 4]
        Freq5 = AECh['P-FRQ'][num + 5]
        
        Time0 = AECh['SSSSSSSS.mmmuuun_y'][num]
        Time1 = AECh['SSSSSSSS.mmmuuun_y'][num + 1]
        Time2 = AECh['SSSSSSSS.mmmuuun_y'][num + 2]
        Time3 = AECh['SSSSSSSS.mmmuuun_y'][num + 3]
        Time4 = AECh['SSSSSSSS.mmmuuun_y'][num + 4]
        Time5 = AECh['SSSSSSSS.mmmuuun_y'][num + 5]
        
        Thr = AECh['THR'][num]
        
        plt.cla()
        plt.clf()
        plt.close()
        fig = plt.figure(figsize=(16,9)  , dpi=72)
        gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
        (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
        for ax in fig.get_axes():
            ax.label_outer()
        
        ax1.plot(t0, V0, 'tab:red')
        ax2.plot(t1, V1, 'tab:orange')
        ax3.plot(t2, V2, 'tab:green')
        ax4.plot(t3, V3, 'tab:blue')
        ax5.plot(t4, V4, 'tab:purple')
        ax6.plot(t5, V5, 'tab:pink')
        
        fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
        fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
        
        fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 3.1, "Viewing hits: {:d} - {:d} | Event: {:d}".format(viewfirst, viewlast, eventnum), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.text(-0.98, 2.08, "Arrival time: {:.8f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.08, "Arrival time: {:.8f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.08, "Arrival time: {:.8f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.08, "Arrival time: {:.8f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.08, "Arrival time: {:.8f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 0.08, "Arrival time: {:.8f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.tight_layout(pad=5)
        
        canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
        canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
            
        toolbarFrame = tk.Frame(master=root)
        toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
        toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
        toolbar.config(bg="#212325")
        toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
        toolbar.winfo_children()[-2].config(background="#212325")
        toolbar.update()
    else:
        num = label_counter.cget("text") * 5
        viewfirst = num + 1
        viewlast = num + 5
        eventnum = int(viewlast / 5)
        
        t0, V0 = MistrasDTA.get_waveform_data(AECh[num])
        t1, V1 = MistrasDTA.get_waveform_data(AECh[num + 1])
        t2, V2 = MistrasDTA.get_waveform_data(AECh[num + 2])
        t3, V3 = MistrasDTA.get_waveform_data(AECh[num + 3])
        t4, V4 = MistrasDTA.get_waveform_data(AECh[num + 4])
        #t5, V5 = MistrasDTA.get_waveform_data(AECh[num + 5])
        
        Ch0 = AECh['CH'][num]
        Ch1 = AECh['CH'][num + 1]
        Ch2 = AECh['CH'][num + 2]
        Ch3 = AECh['CH'][num + 3]
        Ch4 = AECh['CH'][num + 4]
        #Ch5 = AECh['CH'][num + 5]
        
        Amp0 = AECh['AMP'][num]
        Amp1 = AECh['AMP'][num + 1]
        Amp2 = AECh['AMP'][num + 2]
        Amp3 = AECh['AMP'][num + 3]
        Amp4 = AECh['AMP'][num + 4]
        #Amp5 = AECh['AMP'][num + 5]
        
        En0 = AECh['ABS-ENERGY'][num]
        En1 = AECh['ABS-ENERGY'][num + 1]
        En2 = AECh['ABS-ENERGY'][num + 2]
        En3 = AECh['ABS-ENERGY'][num + 3]
        En4 = AECh['ABS-ENERGY'][num + 4]
        #En5 = AECh['ABS-ENERGY'][num + 5]
        
        Count0 = AECh['COUN'][num]
        Count1 = AECh['COUN'][num + 1]
        Count2 = AECh['COUN'][num + 2]
        Count3 = AECh['COUN'][num + 3]
        Count4 = AECh['COUN'][num + 4]
        #Count5 = AECh['COUN'][num + 5]
    
        Freq0 = AECh['P-FRQ'][num]
        Freq1 = AECh['P-FRQ'][num + 1]
        Freq2 = AECh['P-FRQ'][num + 2]
        Freq3 = AECh['P-FRQ'][num + 3]
        Freq4 = AECh['P-FRQ'][num + 4]
        #Freq5 = AECh['P-FRQ'][num + 5]
        
        Time0 = AECh['SSSSSSSS.mmmuuun_y'][num]
        Time1 = AECh['SSSSSSSS.mmmuuun_y'][num + 1]
        Time2 = AECh['SSSSSSSS.mmmuuun_y'][num + 2]
        Time3 = AECh['SSSSSSSS.mmmuuun_y'][num + 3]
        Time4 = AECh['SSSSSSSS.mmmuuun_y'][num + 4]
        #Time5 = AECh['SSSSSSSS.mmmuuun_y'][num + 5]
        
        Thr = AECh['THR'][num]
        
        plt.cla()
        plt.clf()
        plt.close()
        fig = plt.figure(figsize=(16,9)  , dpi=72)
        gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
        (ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots(sharex=True, sharey=True)
        for ax in fig.get_axes():
            ax.label_outer()
        
        ax1.plot(t0, V0, 'tab:red')
        ax2.plot(t1, V1, 'tab:orange')
        ax3.plot(t2, V2, 'tab:green')
        ax4.plot(t3, V3, 'tab:blue')
        ax5.plot(t4, V4, 'tab:purple')
        #ax6.plot(t5, V5, 'tab:pink')
        
        fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
        fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
        
        fig.text(-0.98, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp0, En0, Count0, Freq0, Ch0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp1, En1, Count1, Freq1, Ch1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp2, En2, Count2, Freq2, Ch2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp3, En3, Count3, Freq3, Ch3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp4, En4, Count4, Freq4, Ch4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        #fig.text(0.03, 0.9, "Amp: {:d} dB    |    Abs. Energy: {:0.2e}    |    Cnt: {:d}    |    Freq: {:d}    |    Ch: {:d}".format(Amp5, En5, Count5, Freq5, Ch5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.55, 3.1, "First threshold crossing (FTC): {:d} dB".format(Thr), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 3.1, "Viewing hits: {:d} - {:d} | Event: {:d}".format(viewfirst, viewlast, eventnum), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.text(-0.98, 2.08, "Arrival time: {:.8f} s".format(Time0), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 2.08, "Arrival time: {:.8f} s".format(Time1), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 1.08, "Arrival time: {:.8f} s".format(Time2), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(0.03, 1.08, "Arrival time: {:.8f} s".format(Time3), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        fig.text(-0.98, 0.08, "Arrival time: {:.8f} s".format(Time4), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        #fig.text(0.03, 0.08, "Arrival time: {:.8f} s".format(Time5), fontsize = 12, horizontalalignment='left', verticalalignment='center', transform = ax.transAxes)
        
        fig.tight_layout(pad=5)
        
        canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
        canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
            
        toolbarFrame = tk.Frame(master=root)
        toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
        toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
        toolbar.config(bg="#212325")
        toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
        toolbar.winfo_children()[-2].config(background="#212325")
        toolbar.update()        
 
    return

#-- GUI ROW BY ROW --# 
#Discard channel #
Chnum = tk.StringVar()
entry_Ch = customtkinter.CTkEntry(master=root, textvariable=Chnum, border_width = 2, width = 100, justify='center')
entry_Ch.grid(row=0, column=2, sticky=tk.W, padx=(45, 5), pady=(15,5)) 
entry_Ch.insert(0, '0')
label_Channel = tk.Label(root, text="Enter Ch # to remove ('0' is none):")
label_Channel.grid(row=0, column=0, sticky=tk.W, columnspan=6, pady=(10,0), padx=(10, 0))
label_Channel.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')

#Precision level
prec = tk.StringVar()
entry_prec = customtkinter.CTkEntry(master=root, textvariable=prec, border_width = 2, width = 100, justify='center')
entry_prec.grid(row=0, column=9, sticky=tk.W, padx=(0, 5), pady=(15,5)) 
entry_prec.insert(0, '6')
label_Precision = tk.Label(root, text="Arrival time precision (# of decimal places, max 8):")
label_Precision.grid(row=0, column=3, sticky=tk.W, columnspan=6, pady=(10,0), padx=(10, 0))
label_Precision.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')

#Row 1
label_RunNumber = tk.Label(root, text="Then load a Mistras DTA file and view the AE waveforms")
label_RunNumber.grid(row=1, column=0, sticky=tk.W, columnspan=10, pady=(10,10), padx=(10, 0))
label_RunNumber.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')

label_loadDTA = tk.Label(root, text="path") #redirect for another function
label_loadpath = tk.Label(root, text="path") #redirect for another function

label_hits = tk.Label(root, text="Total hits:", width = 9)
label_hits.grid(row=2, column=1, sticky=tk.E)
label_hits.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')
label_numhits = tk.Label(root, text="    ", width = 4)
label_numhits.grid(row=2, column=2, sticky=tk.W)
label_numhits.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')

label_events = tk.Label(root, text="Total events:", width = 9)
label_events.grid(row=2, column=2, sticky=tk.E)
label_events.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')
label_numevents = tk.Label(root, text="    ", width = 3)
label_numevents.grid(row=2, column=3, sticky=tk.W)
label_numevents.config(bg="#212325", fg="#FFFFFF", font='Helvetica 11')

#-- Button --#
loadDTA = partial(loadfile, label_loadDTA, label_loadpath)
extract = partial(extractall, label_numhits, label_numevents)
buttonCal = customtkinter.CTkButton(root, text="Load DTA file", width = 100, command=lambda:[loadDTA(),_clear(),extract()])
buttonCal.grid(row=2, column=0, sticky=tk.W, padx=(15, 5), pady=(2,2))

#Counter
label_counter = tk.Label(root, textvariable=counter)
label_counter.grid(row=2, column=5)
label_counter.config(bg="#212325", fg="#FFFFFF", font='Helvetica 12 bold')
counterbutton = customtkinter.CTkButton(root, text=u"\u2190", width = 40, command=lambda:[onClickdec(),_clear(),arrow()])
counterbutton.grid(row=2, column=4, sticky=tk.E, padx=(0, 0))
counterbutton = customtkinter.CTkButton(root, text=u"\u2192", width = 40, command=lambda:[onClick(),_clear(),arrow()])
counterbutton.grid(row=2, column=6, sticky=tk.W)
counterbutton = customtkinter.CTkButton(root, text="Reset", width = 60, command=lambda:[reset(),_clear(),arrow()])
counterbutton.grid(row=2, column=7)

goto = tk.StringVar()
entry_goto = customtkinter.CTkEntry(master=root, textvariable=goto, border_width = 2, width = 50, justify='center')
entry_goto.grid(row=2, column=8, sticky=tk.E, padx=(0, 2), pady=(5,5)) 
entry_goto.insert(0, '0')
counterbutton = customtkinter.CTkButton(root, text="Go to", width = 60, command=lambda:[gotobutton(),_clear(),arrow()])
counterbutton.grid(row=2, column=9, sticky=tk.W)

#blank line
label_Import = tk.Label(root, text="")
label_Import.grid(row=3, column=0, sticky=tk.W, columnspan=10)
label_Import.config(bg="#212325", fg="#FFFFFF")

#Draw plot area
fig = plt.figure(figsize=(16,9)  , dpi=72)
gs = fig.add_gridspec(3, 2, hspace=0, wspace=0)
(ax1, ax2), (ax3, ax4), (ax5, ax6) = gs.subplots()
for ax in fig.get_axes():
    ax.label_outer()
fig.text(0.5, 0.04, 'Time (microseconds)', ha='center', fontsize=14)
fig.text(0.02, 0.5, 'Voltage (v)', va='center', rotation='vertical', fontsize=14)
fig.tight_layout(pad=5)

canvas = FigureCanvasTkAgg(fig, master=root)  # A tk.DrawingArea.
canvas.get_tk_widget().grid(row=4, column=0, columnspan=15, rowspan=1, padx=(10,10))
    
toolbarFrame = tk.Frame(master=root)
toolbarFrame.grid(row=5, column=1, columnspan=15, rowspan=1, sticky=tk.W, padx=(330,0))
toolbar = NavigationToolbar2Tk(canvas, toolbarFrame)
toolbar.config(bg="#212325")
toolbar._message_label.config(bg="#212325", fg="#FFFFFF")
toolbar.winfo_children()[-2].config(background="#212325")
toolbar.update()

label_Note = tk.Label(root, text="Written by R. Farla at P61B    20.12.2023    v1.1")
label_Note.grid(row=5, column=9, columnspan=6, pady=(15,5))
label_Note.config(bg="#212325", fg="#FFFFFF", font='Helvetica 9 bold')

root.mainloop()